/*
 * Created on 01-jul-2005
 *
 * gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.gvsig.geoprocessing.gui;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;

import com.hardcode.gdbms.engine.data.DataSource;
import com.hardcode.gdbms.engine.values.Value;
import com.iver.andami.PluginServices;
import com.iver.andami.ui.mdiManager.View;
import com.iver.andami.ui.mdiManager.ViewInfo;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.core.FPolygon2D;
import com.iver.cit.gvsig.fmap.core.FShape;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.ShapeFactory;
import com.iver.cit.gvsig.fmap.core.v02.FConverter;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.layers.FBitSet;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLayers;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.fmap.layers.VectorialAdapter;
import com.iver.cit.gvsig.fmap.layers.layerOperations.AlphanumericData;
import com.iver.cit.gvsig.fmap.layers.layerOperations.Selectable;
import com.iver.cit.gvsig.fmap.layers.layerOperations.SingleLayer;
import com.iver.cit.gvsig.fmap.layers.layerOperations.VectorialData;
import com.iver.cit.gvsig.fmap.operations.strategies.SelectedShapeVisitor;
import com.iver.cit.gvsig.fmap.operations.strategies.SelectedZoomVisitor;
import com.iver.cit.gvsig.fmap.operations.strategies.VisitException;
import com.iver.gvsig.geoprocessing.gui.operationpanels.GeoProcessingBufferPanel;
import com.iver.gvsig.geoprocessing.gui.operationpanels.GeoProcessingClipPanel;
import com.iver.gvsig.geoprocessing.gui.operationpanels.GeoProcessingDissolvePanel;
import com.iver.gvsig.geoprocessing.gui.operationpanels.GeoProcessingIntersectPanel;
import com.iver.gvsig.geoprocessing.gui.operationpanels.GeoProcessingMergePanel;
import com.iver.gvsig.geoprocessing.gui.operationpanels.GeoProcessingSpatialjoinPanel;
import com.iver.gvsig.geoprocessing.gui.operationpanels.GeoProcessingUnionPanel;
import com.iver.gvsig.geoprocessing.operations.GeoProcessingPersistenceOperations;
import com.iver.gvsig.geoprocessing.operations.GeoProcessingTopologyOperations;
import com.iver.gvsig.geoprocessing.operations.strategies.SelectedShapeToJTSVisitor;
import com.vividsolutions.jts.geom.Geometry;

import java.awt.Component;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.FlowLayout;
import java.awt.CardLayout;
import javax.swing.BoxLayout;
import java.awt.BorderLayout;
import javax.swing.JButton;

public class GeoProcessingPanel extends JPanel implements View {

    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    private GeoProcessingOperationSelectorPanel geoProcessingOperationSelectorPanel = null;
    private GeoProcessingBufferPanel geoProcessingBufferPanel = null;
    private GeoProcessingClipPanel geoProcessingClipPanel = null;
    private GeoProcessingDissolvePanel geoProcessingDissolvePanel = null;
    private GeoProcessingMergePanel geoProcessingMergePanel = null;
    private GeoProcessingIntersectPanel geoProcessingIntersectPanel = null;
    private GeoProcessingUnionPanel geoProcessingUnionPanel = null;
    private GeoProcessingSpatialjoinPanel geoProcessingSpatialjoinPanel = null;
    private JPanel buttonsPanel = null;

	private JButton jButton = null;
	private JButton jButton1 = null;
	private JButton jButton2 = null;
    private ViewInfo viewInfo = null;
    private FLayers layers = null;
    /**
     * This constructor initializes the set of layers
     */
    public GeoProcessingPanel(FLayers layers) {
        super();
        this.layers = layers;
        initialize();
    }

    /**
     * This method initializes this
     * 
     * @return void
     */
    private void initialize() {
        this.setLayout(new BorderLayout());
        this.setSize(416, 271);
        this.add(getButtonsPanel(), java.awt.BorderLayout.SOUTH);
        this.add(getGeoProcessingOperationSelectorPanel(), java.awt.BorderLayout.NORTH);
        this.add(getGeoProcessingBufferPanel(), java.awt.BorderLayout.NORTH);
        this.add(getGeoProcessingClipPanel(), java.awt.BorderLayout.NORTH);
        this.add(getGeoProcessingDissolvePanel(), java.awt.BorderLayout.NORTH);
        this.add(getGeoProcessingMergePanel(), java.awt.BorderLayout.NORTH);
        this.add(getGeoProcessingIntersectPanel(), java.awt.BorderLayout.NORTH);
        this.add(getGeoProcessingUnionPanel(), java.awt.BorderLayout.NORTH);
        this.add(getGeoProcessingSpatialjoinPanel(), java.awt.BorderLayout.NORTH);
        geoProcessingOperationSelectorPanel.setVisible(true);
        geoProcessingBufferPanel.setVisible(false);
        geoProcessingClipPanel.setVisible(false);
        geoProcessingDissolvePanel.setVisible(false);
        geoProcessingMergePanel.setVisible(false);
        geoProcessingIntersectPanel.setVisible(false);
        geoProcessingUnionPanel.setVisible(false);
        geoProcessingSpatialjoinPanel.setVisible(false);
        jButton1.setEnabled(false);
    }

    public ViewInfo getViewInfo() {
        // TODO Auto-generated method stub
        if (viewInfo == null) {
            viewInfo=new ViewInfo(ViewInfo.MODALDIALOG);
            viewInfo.setTitle("GeoProcessing");
        }
        return viewInfo;
    }

    /**
     * This method initializes geoProcessingOperationSelectorPanel	
     * 	
     * @return javax.swing.JPanel	
     */    
    private JPanel getGeoProcessingOperationSelectorPanel() {
    	if (geoProcessingOperationSelectorPanel == null) {
    		geoProcessingOperationSelectorPanel = new GeoProcessingOperationSelectorPanel();
    		geoProcessingOperationSelectorPanel.setName("geoProcessingOperationSelectorPanel");
    	}
    	return geoProcessingOperationSelectorPanel;
    }

    /**
     * This method initializes geoProcessingBufferPanel	
     * 	
     * @return javax.swing.JPanel	
     */    
    private JPanel getGeoProcessingBufferPanel() {
    	if (geoProcessingBufferPanel == null) {
    		geoProcessingBufferPanel = new GeoProcessingBufferPanel(layers);
    		geoProcessingBufferPanel.setName("geoProcessingBufferPanel");
    	}
    	return geoProcessingBufferPanel;
    }

    /**
     * This method initializes geoProcessingClipPanel	
     * 	
     * @return javax.swing.JPanel	
     */    
    private JPanel getGeoProcessingClipPanel() {
    	if (geoProcessingClipPanel == null) {
    		geoProcessingClipPanel = new GeoProcessingClipPanel(layers);
    		geoProcessingClipPanel.setName("geoProcessingClipPanel");
    		// Si no le meto esta lnea, no se visualiza el men. Ver que puede
            // estar pasando ...
            geoProcessingClipPanel.setPreferredSize(new java.awt.Dimension(300,300));
    	}
    	return geoProcessingClipPanel;
    }

    /**
     * This method initializes geoProcessingDissolvePanel 
     *  
     * @return javax.swing.JPanel   
     */    
    private JPanel getGeoProcessingDissolvePanel() {
        if (geoProcessingDissolvePanel == null) {
            geoProcessingDissolvePanel = new GeoProcessingDissolvePanel(layers);
            geoProcessingDissolvePanel.setName("geoProcessingDissolvePanel");
        }
        return geoProcessingDissolvePanel;
    }

    /**
     * This method initializes geoProcessingMergePanel 
     *  
     * @return javax.swing.JPanel   
     */    
    private JPanel getGeoProcessingMergePanel() {
        if (geoProcessingMergePanel == null) {
            geoProcessingMergePanel = new GeoProcessingMergePanel(layers);
            geoProcessingMergePanel.setName("geoProcessingMergePanel");
        }
        return geoProcessingMergePanel;
    }

    /**
     * This method initializes geoProcessingIntersectPanel 
     *  
     * @return javax.swing.JPanel   
     */    
    private JPanel getGeoProcessingIntersectPanel() {
        if (geoProcessingIntersectPanel == null) {
            geoProcessingIntersectPanel = new GeoProcessingIntersectPanel(layers);
            geoProcessingIntersectPanel.setName("geoProcessingIntersectPanel");
        }
        return geoProcessingIntersectPanel;
    }

    /**
     * This method initializes geoProcessingUnionPanel 
     *  
     * @return javax.swing.JPanel   
     */    
    private JPanel getGeoProcessingUnionPanel() {
        if (geoProcessingUnionPanel == null) {
            geoProcessingUnionPanel = new GeoProcessingUnionPanel(layers);
            geoProcessingUnionPanel.setName("geoProcessingUnionPanel");
        }
        return geoProcessingUnionPanel;
    }

    /**
     * This method initializes geoProcessingSpatialjoinPanel 
     *  
     * @return javax.swing.JPanel   
     */    
    private JPanel getGeoProcessingSpatialjoinPanel() {
        if (geoProcessingSpatialjoinPanel == null) {
            geoProcessingSpatialjoinPanel = new GeoProcessingSpatialjoinPanel(layers);
            geoProcessingSpatialjoinPanel.setName("geoProcessingSpatialjoinPanel");
            // Si no le meto esta lnea, no se visualiza el men. Ver que puede
            // estar pasando ...
            geoProcessingSpatialjoinPanel.setPreferredSize(new java.awt.Dimension(300,300));
        }
        return geoProcessingSpatialjoinPanel;
    }

    /**
     * This method initializes buttonsPanel	
     * 	
     * @return javax.swing.JPanel	
     */    
    private JPanel getButtonsPanel() {
    	if (buttonsPanel == null) {
    		buttonsPanel = new JPanel();
    		buttonsPanel.setName("buttonsPanel");
    		buttonsPanel.setBorder(javax.swing.BorderFactory.createEtchedBorder(javax.swing.border.EtchedBorder.LOWERED));
    		buttonsPanel.add(getJButton(), null);
    		buttonsPanel.add(getJButton1(), null);
    		buttonsPanel.add(getJButton2(), null);
    	}
    	return buttonsPanel;
    }

	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJButton() {
		if (jButton == null) {
			jButton = new JButton();
			jButton.setText("Cerrar");
			jButton.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					System.out.println("actionPerformed()"); // TODO Auto-generated Event stub actionPerformed()
                    closeMethod();
				}
			});
		}
		return jButton;
	}
	/**
	 * This method initializes jButton1	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJButton1() {
		if (jButton1 == null) {
			jButton1 = new JButton();
			jButton1.setText("Anterior");
			jButton1.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					System.out.println("actionPerformed()"); // TODO Auto-generated Event stub actionPerformed()
                    getGeoProcessingOperationSelectorPanel().setVisible(true);
                    getGeoProcessingBufferPanel().setVisible(false);
                    getGeoProcessingClipPanel().setVisible(false);
                    getGeoProcessingDissolvePanel().setVisible(false);
                    getGeoProcessingMergePanel().setVisible(false);
                    getGeoProcessingIntersectPanel().setVisible(false);
                    getGeoProcessingUnionPanel().setVisible(false);
                    getGeoProcessingSpatialjoinPanel().setVisible(false);
                    jButton1.setEnabled(false);
                    jButton2.setText("Siguiente");
				}
			});
		}
		return jButton1;
	}
	/**
	 * This method initializes jButton2	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJButton2() {
		if (jButton2 == null) {
			jButton2 = new JButton();
			jButton2.setText("Siguiente");
			jButton2.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					System.out.println("actionPerformed()"); // TODO Auto-generated Event stub actionPerformed()
					if (jButton2.getText().equals("Siguiente")) {
                        if (((JRadioButton)geoProcessingOperationSelectorPanel.getComponent(1)).isSelected()) {
                            getGeoProcessingOperationSelectorPanel().setVisible(false);
                            getGeoProcessingBufferPanel().setVisible(true);
                            getGeoProcessingClipPanel().setVisible(false);
                            getGeoProcessingDissolvePanel().setVisible(false);
                            getGeoProcessingMergePanel().setVisible(false);
                            getGeoProcessingIntersectPanel().setVisible(false);
                            getGeoProcessingUnionPanel().setVisible(false);
                            getGeoProcessingSpatialjoinPanel().setVisible(false);
                        } else if (((JRadioButton)geoProcessingOperationSelectorPanel.getComponent(2)).isSelected()) {
                            getGeoProcessingOperationSelectorPanel().setVisible(false);
                            getGeoProcessingBufferPanel().setVisible(false);
                            getGeoProcessingClipPanel().setVisible(true);
                            getGeoProcessingDissolvePanel().setVisible(false);
                            getGeoProcessingMergePanel().setVisible(false);
                            getGeoProcessingIntersectPanel().setVisible(false);
                            getGeoProcessingUnionPanel().setVisible(false);
                            getGeoProcessingSpatialjoinPanel().setVisible(false);
                        } else if (((JRadioButton)geoProcessingOperationSelectorPanel.getComponent(3)).isSelected()) {
                            getGeoProcessingOperationSelectorPanel().setVisible(false);
                            getGeoProcessingBufferPanel().setVisible(false);
                            getGeoProcessingClipPanel().setVisible(false);
                            getGeoProcessingDissolvePanel().setVisible(true);
                            getGeoProcessingMergePanel().setVisible(false);
                            getGeoProcessingIntersectPanel().setVisible(false);
                            getGeoProcessingUnionPanel().setVisible(false);
                            getGeoProcessingSpatialjoinPanel().setVisible(false);
                        } else if (((JRadioButton)geoProcessingOperationSelectorPanel.getComponent(4)).isSelected()) {
                            getGeoProcessingOperationSelectorPanel().setVisible(false);
                            getGeoProcessingBufferPanel().setVisible(false);
                            getGeoProcessingClipPanel().setVisible(false);
                            getGeoProcessingDissolvePanel().setVisible(false);
                            getGeoProcessingMergePanel().setVisible(true);
                            getGeoProcessingIntersectPanel().setVisible(false);
                            getGeoProcessingUnionPanel().setVisible(false);
                            getGeoProcessingSpatialjoinPanel().setVisible(false);
                        } else if (((JRadioButton)geoProcessingOperationSelectorPanel.getComponent(5)).isSelected()) {
                            getGeoProcessingOperationSelectorPanel().setVisible(false);
                            getGeoProcessingBufferPanel().setVisible(false);
                            getGeoProcessingClipPanel().setVisible(false);
                            getGeoProcessingDissolvePanel().setVisible(false);
                            getGeoProcessingMergePanel().setVisible(false);
                            getGeoProcessingIntersectPanel().setVisible(true);
                            getGeoProcessingUnionPanel().setVisible(false);
                            getGeoProcessingSpatialjoinPanel().setVisible(false);
                        } else if (((JRadioButton)geoProcessingOperationSelectorPanel.getComponent(6)).isSelected()) {
                            getGeoProcessingOperationSelectorPanel().setVisible(false);
                            getGeoProcessingBufferPanel().setVisible(false);
                            getGeoProcessingClipPanel().setVisible(false);
                            getGeoProcessingDissolvePanel().setVisible(false);
                            getGeoProcessingMergePanel().setVisible(false);
                            getGeoProcessingIntersectPanel().setVisible(false);
                            getGeoProcessingUnionPanel().setVisible(true);
                            getGeoProcessingSpatialjoinPanel().setVisible(false);
                        } else if (((JRadioButton)geoProcessingOperationSelectorPanel.getComponent(7)).isSelected()) {
                            getGeoProcessingOperationSelectorPanel().setVisible(false);
                            getGeoProcessingBufferPanel().setVisible(false);
                            getGeoProcessingClipPanel().setVisible(false);
                            getGeoProcessingDissolvePanel().setVisible(false);
                            getGeoProcessingMergePanel().setVisible(false);
                            getGeoProcessingIntersectPanel().setVisible(false);
                            getGeoProcessingUnionPanel().setVisible(false);
                            getGeoProcessingSpatialjoinPanel().setVisible(true);
                        } else {
                            // Si no hay ninguna operacin de geoprocesamiento seleccionada no se cambia el dilogo de opciones.
                            getGeoProcessingOperationSelectorPanel().setVisible(true);
                            getGeoProcessingBufferPanel().setVisible(false);
                            getGeoProcessingClipPanel().setVisible(false);
                            getGeoProcessingDissolvePanel().setVisible(false);
                            getGeoProcessingMergePanel().setVisible(false);
                            getGeoProcessingIntersectPanel().setVisible(false);
                            getGeoProcessingUnionPanel().setVisible(false);
                            getGeoProcessingSpatialjoinPanel().setVisible(false);
                        }
                        jButton1.setEnabled(true);
                        jButton2.setText("Terminar");
                    } else if (jButton2.getText().equals("Terminar")) {
                        if (((JRadioButton)geoProcessingOperationSelectorPanel.getComponent(1)).isSelected()) {
                            bufferMethod();
                        } else if (((JRadioButton)geoProcessingOperationSelectorPanel.getComponent(2)).isSelected()) {
                            clipMethod();
                        } else if (((JRadioButton)geoProcessingOperationSelectorPanel.getComponent(3)).isSelected()) {
                            dissolveMethod();
                        } else if (((JRadioButton)geoProcessingOperationSelectorPanel.getComponent(4)).isSelected()) {
                            mergeMethod();
                        } else if (((JRadioButton)geoProcessingOperationSelectorPanel.getComponent(5)).isSelected()) {
                            intersectMethod();
                        } else if (((JRadioButton)geoProcessingOperationSelectorPanel.getComponent(6)).isSelected()) {
                            unionMethod();
                        } else if (((JRadioButton)geoProcessingOperationSelectorPanel.getComponent(7)).isSelected()) {
                            spatialjoinMethod();
                        } else {
                            //
                        }
                        closeMethod();
                    }
				}
			});
		}
		return jButton2;
	}
    private Geometry[] getJtsGeometries(FLayer bufferingLayer, Component component) {
        FBitSet fBitSet = ((Selectable)bufferingLayer).getSelection();
        // Extraigo todos los elementos de la capa o solo los seleccionados
        if (!((JCheckBox)component).isSelected()) {
            VectorialAdapter va = (VectorialAdapter)((SingleLayer)bufferingLayer).getSource();
            fBitSet = new FBitSet();
            try {
                for (int i=0;i<va.getShapeCount();i++) {
                   fBitSet.set(i);
                }
            } catch (DriverIOException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
            ((Selectable)bufferingLayer).setSelection(fBitSet);
        }
        SelectedShapeToJTSVisitor visitor = new SelectedShapeToJTSVisitor();
        try {
            ((VectorialData)bufferingLayer).process(visitor);
        } catch (DriverException e1) {
            // TODO Auto-generated catch block
            e1.printStackTrace();
        } catch (VisitException e1) {
            // TODO Auto-generated catch block
            e1.printStackTrace();
        }
        // Almaceno en un vector los elementos para el buffer
        return visitor.getJtsGeometries();
    }
    private IGeometry[] getFmapGeometries(Geometry[] jtsGeometries) {
        FShape[] shapes = new FShape[jtsGeometries.length];
        IGeometry[] geoms = new IGeometry[jtsGeometries.length];
        for (int i=0;i<jtsGeometries.length;i++) {
            shapes[i] = FConverter.jts_to_java2d(jtsGeometries[i]);
            geoms[i] = ShapeFactory.createGeometry(shapes[i]);
        }
        return geoms;
    }
    private void bufferMethod() {
        String bufferingLayerName = (String)((JComboBox)geoProcessingBufferPanel.getComponent(2)).getSelectedItem();
        FLayer bufferingLayer = layers.getLayer(bufferingLayerName);
        Geometry[] geometries = getJtsGeometries(bufferingLayer, geoProcessingBufferPanel.getComponent(3));
        IGeometry[] shapes = null;
        // Realizo el buffer llamando al metodo makeBuffer de GeoProcessingTopologyOperations
        if (((JRadioButton)geoProcessingBufferPanel.getComponent(4)).isSelected()) {
            double bufferDistance = Double.parseDouble(((JTextField)geoProcessingBufferPanel.getComponent(7)).getText());
            if (((JCheckBox)geoProcessingBufferPanel.getComponent(13)).isSelected()) {
                Geometry jtsBuffer = GeoProcessingTopologyOperations.makeDissolveBuffer(geometries, bufferDistance, getQuadrantSegments());
                // Realizo las operaciones relacionadas con tablas que el buffer requiere llamando a los mtodos convenientes de GeoProcessingTableOperations
                // (A lo mejor este paso no es necesario) Convierto las tablas modificadas y las nuevas entidades JTS al modelo de gvSIG
                System.out.println("jtsBuffer = " + jtsBuffer);
                FPolygon2D shapeBuffer = (FPolygon2D)FConverter.jts_to_java2d(jtsBuffer);
                System.out.println("shapeBuffer = " + shapeBuffer);
                shapes = new IGeometry[]{ShapeFactory.createGeometry(shapeBuffer)};
                // Guardo la tabla mediante gdbms. Guardo tambin las entidades JTS mediante gvSIG o usando solo gdbms (probablemente bastar con usar gdbms)
            } else {
                double[] bufferDistances = new double[geometries.length];
                for (int i=0;i<bufferDistances.length;i++) {
                    bufferDistances[i] = bufferDistance;
                }
                Geometry[] jtsBuffers = GeoProcessingTopologyOperations.makeBuffers(geometries, bufferDistances, getQuadrantSegments());
                // Realizo las operaciones relacionadas con tablas que el buffer requiere llamando a los mtodos convenientes de GeoProcessingTableOperations
                // (A lo mejor este paso no es necesario) Convierto las tablas modificadas y las nuevas entidades JTS al modelo de gvSIG
                shapes = getFmapGeometries(jtsBuffers);
                // Guardo la tabla mediante gdbms. Guardo tambin las entidades JTS mediante gvSIG o usando solo gdbms (probablemente bastar con usar gdbms)
            }
        } else if (((JRadioButton)geoProcessingBufferPanel.getComponent(5)).isSelected()) {
            double[] bufferDistances = getBufferDistancesFromTable(bufferingLayer);
            if (((JCheckBox)geoProcessingBufferPanel.getComponent(13)).isSelected()) {
                // Aqu hay problemas ... Con distintas distancias y haciendo dissolve ...
                Geometry jtsBuffer = GeoProcessingTopologyOperations.makeDissolveBuffer(geometries, bufferDistances, getQuadrantSegments());
                // Realizo las operaciones relacionadas con tablas que el buffer requiere llamando a los mtodos convenientes de GeoProcessingTableOperations
                // (A lo mejor este paso no es necesario) Convierto las tablas modificadas y las nuevas entidades JTS al modelo de gvSIG
                System.out.println("jtsBuffer = " + jtsBuffer);
                FPolygon2D shapeBuffer = (FPolygon2D)FConverter.jts_to_java2d(jtsBuffer);
                System.out.println("shapeBuffer = " + shapeBuffer);
                shapes = new IGeometry[]{ShapeFactory.createGeometry(shapeBuffer)};
                // Guardo la tabla mediante gdbms. Guardo tambin las entidades JTS mediante gvSIG o usando solo gdbms (probablemente bastar con usar gdbms)
            } else {
                Geometry[] jtsBuffers = GeoProcessingTopologyOperations.makeBuffers(geometries, bufferDistances, getQuadrantSegments());
                // Realizo las operaciones relacionadas con tablas que el buffer requiere llamando a los mtodos convenientes de GeoProcessingTableOperations
                // (A lo mejor este paso no es necesario) Convierto las tablas modificadas y las nuevas entidades JTS al modelo de gvSIG
                shapes = getFmapGeometries(jtsBuffers);
                // Guardo la tabla mediante gdbms. Guardo tambin las entidades JTS mediante gvSIG o usando solo gdbms (probablemente bastar con usar gdbms)
            }
        }
        GeoProcessingPersistenceOperations.saveResultsToShapeFile(shapes, geoProcessingBufferPanel.getOutputFile());
        if (!((JCheckBox)geoProcessingBufferPanel.getComponent(3)).isSelected()) ((Selectable)bufferingLayer).clearSelection();
    }
    private void clipMethod() {
        String layerToClipName = (String)((JComboBox)geoProcessingClipPanel.getComponent(0)).getSelectedItem();
        FLayer layerToClip = layers.getLayer(layerToClipName);
        Geometry[] geometriesToClip = getJtsGeometries(layerToClip, geoProcessingClipPanel.getComponent(1));
        String clippingLayerName = (String)((JComboBox)geoProcessingClipPanel.getComponent(3)).getSelectedItem();
        FLayer clippingLayer = layers.getLayer(clippingLayerName);
        Geometry[] clippingGeometries = getJtsGeometries(clippingLayer, geoProcessingClipPanel.getComponent(5));
        Geometry[] clippedGeometries = GeoProcessingTopologyOperations.makeClip(geometriesToClip, clippingGeometries);
        IGeometry[] shapes = getFmapGeometries(clippedGeometries);
        // Guardo la tabla mediante gdbms. Guardo tambin las entidades JTS mediante gvSIG o usando solo gdbms (probablemente bastar con usar gdbms)
        GeoProcessingPersistenceOperations.saveResultsToShapeFile(shapes, geoProcessingClipPanel.getOutputFile());
    }
    private void dissolveMethod() {
        
    }
    private void mergeMethod() {
        
    }
    private void intersectMethod() {
        
    }
    private void unionMethod() {
        
    }
    private void spatialjoinMethod() {
        
    }
    public double[] getBufferDistancesFromTable(FLayer bufferingLayer) {
        AlphanumericData lyr = (AlphanumericData)bufferingLayer;
        DataSource ds;
        double[] bufferDistances = null;
        try {
            ds = lyr.getRecordset();
            int fieldId = ds.getFieldIndexByName(getSelectedField());
            Value[] fieldValues = new Value[(int)ds.getRowCount()];
            SelectableDataSource selectableDataSource = (SelectableDataSource)ds;
            FBitSet fBitSet = selectableDataSource.getSelection();
            Value[] valuesSelected = new Value[fBitSet.cardinality()];
            int j = 0;
            for (int i=0;i<ds.getRowCount();i++) {
                boolean isSelected = fBitSet.get(i);
                fieldValues[i] = ds.getFieldValue(i, fieldId);
                if (isSelected) {
                    valuesSelected[j] = fieldValues[i];
                    j++;
                }
            }
            bufferDistances = new double[fBitSet.cardinality()];
            for (int i=0;i<fBitSet.cardinality();i++) {
                bufferDistances[i] = new Double(valuesSelected[i].toString()).doubleValue();
            }
        } catch (DriverException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return bufferDistances;
    }
    private String getSelectedField() {
        return ((String)((JComboBox)geoProcessingBufferPanel.getComponent(6)).getSelectedItem());
    }
    private int getQuadrantSegments() {
        return (Integer.valueOf((String)((JComboBox)geoProcessingBufferPanel.getComponent(11)).getSelectedItem())).intValue();
    }
    private void closeMethod() {
        if (PluginServices.getMainFrame() == null)
            ((JDialog) (getParent().getParent().getParent().getParent())).dispose();
        else
            PluginServices.getMDIManager().closeView(GeoProcessingPanel.this);
    }
   }  //  @jve:decl-index=0:visual-constraint="10,10"
